package org.itqop.hubmcutils.command.list;

import com.mojang.brigadier.Command;
import com.mojang.brigadier.CommandDispatcher;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.suggestion.Suggestions;
import com.mojang.brigadier.suggestion.SuggestionsBuilder;
import net.minecraft.ChatFormatting;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.Commands;
import net.minecraft.network.chat.ClickEvent;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.HoverEvent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.damagesource.DamageSources;

import java.time.Instant;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

public class PlayerActions {

    private static final Set<String> pendingConfirmation = new HashSet<>();
    private static DamageSources damageSources;
    private static final Map<String, Instant> lastConfirmTime = new HashMap<>();
    private static final long CONFIRM_COOLDOWN_SECONDS = 120;

    public static void registerCommands(CommandDispatcher<CommandSourceStack> dispatcher) {
        dispatcher.register(Commands.literal("suicide")
                .executes(context -> {
                    ServerPlayer player = context.getSource().getPlayerOrException();
                    String playerName = player.getName().getString();

                    pendingConfirmation.add(playerName);

                    Component confirmMessage = Component.literal("[Нажмите здесь, чтобы подтвердить]")
                            .withStyle(style -> style
                                    .withColor(ChatFormatting.RED) // Красный цвет
                                    .withClickEvent(new ClickEvent(ClickEvent.Action.RUN_COMMAND, "/suicide confirm"))
                                    .withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, Component.literal("Подтвердите команду /suicide confirm"))));

                    player.sendSystemMessage(Component.literal("Вы уверены, что хотите покончить с собой? ").append(confirmMessage));
                    return Command.SINGLE_SUCCESS;
                })
                .then(Commands.literal("confirm")
                        .executes(context -> {
                            ServerPlayer player = context.getSource().getPlayerOrException();
                            String playerName = player.getName().getString();

                            Instant now = Instant.now();
                            if (lastConfirmTime.containsKey(playerName)) {
                                Instant lastTime = lastConfirmTime.get(playerName);
                                long secondsSinceLastConfirm = now.getEpochSecond() - lastTime.getEpochSecond();

                                if (secondsSinceLastConfirm < CONFIRM_COOLDOWN_SECONDS) {
                                    long secondsLeft = CONFIRM_COOLDOWN_SECONDS - secondsSinceLastConfirm;
                                    player.sendSystemMessage(Component.literal("Вы не можете использовать команду /suicide confirm прямо сейчас. Подождите еще " + secondsLeft + " секунд.")
                                            .withStyle(ChatFormatting.YELLOW));
                                    return Command.SINGLE_SUCCESS;
                                }
                            }

                            if (pendingConfirmation.contains(playerName)) {
                                if (damageSources == null) {
                                    damageSources = new DamageSources(player.server.registryAccess());
                                }
                                pendingConfirmation.remove(playerName);
                                player.sendSystemMessage(Component.literal("Вы приняли решение покончить с собой.").withStyle(ChatFormatting.RED));
                                player.hurt(damageSources.indirectMagic(player, null), Float.MAX_VALUE);
                                player.closeContainer();
                                lastConfirmTime.put(playerName, now);
                            } else {
                                player.sendSystemMessage(Component.literal("Сначала используйте команду /suicide, чтобы запросить подтверждение.")
                                        .withStyle(ChatFormatting.YELLOW));
                            }
                            return Command.SINGLE_SUCCESS;
                        })
                )
        );

        dispatcher.register(Commands.literal("silenceKill")
                .requires(source -> source.hasPermission(2))
                .then(Commands.argument("target", StringArgumentType.string())
                        .suggests((context, builder) -> suggestOnlinePlayers(context.getSource(), builder)) // Автодополнение игроков
                        .executes(context -> {
                            String targetName = StringArgumentType.getString(context, "target");
                            ServerPlayer targetPlayer = context.getSource().getServer().getPlayerList().getPlayerByName(targetName);

                            if (targetPlayer != null) {
                                targetPlayer.setHealth(0);
                                context.getSource().sendSuccess(
                                        () -> Component.literal("Игрок " + targetName + " беззвучно исчез.").withStyle(ChatFormatting.GREEN), true);
                            } else {
                                context.getSource().sendFailure(
                                        Component.literal("Игрок с ником " + targetName + " не найден.").withStyle(ChatFormatting.RED));
                            }
                            return Command.SINGLE_SUCCESS;
                        })
                )
        );
    }

    private static CompletableFuture<Suggestions> suggestOnlinePlayers(CommandSourceStack source, SuggestionsBuilder builder) {
        source.getServer().getPlayerList().getPlayers().forEach(player ->
                builder.suggest(player.getName().getString())
        );
        return builder.buildFuture();
    }
}
